#ifndef __SOUND_HDSPM_H
#define __SOUND_HDSPM_H
/*
 *   Copyright (C) 2003 Winfried Ritsch (IEM)
 *   based on hdsp.h from Thomas Charbonnel (thomas@undata.org)
 *                      
 *    
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* Maximum channels is 64 even on 56Mode you have 64playbacks to matrix */
#define HDSPM_MAX_CHANNELS      64

enum HDSPM_IO_Type {
  MADI,
  MADIface,
  AIO,
  AES32,
  RayDAT
};


/* -------------------- IOCTL Peak/RMS Meters -------------------- */

struct hdspm_peak_rms {
        uint32_t input_peaks[64];
        uint32_t playback_peaks[64];
        uint32_t output_peaks[64];
  
        uint64_t input_rms[64];
        uint64_t playback_rms[64];
        uint64_t output_rms[64];

        enum {ss, ds, qs} speed;
        int status2;
};


#define SNDRV_HDSPM_IOCTL_GET_PEAK_RMS \
	_IOR('H', 0x40, struct hdspm_peak_rms)

/* ------------ CONFIG block IOCTL ---------------------- */

struct hdspm_config_info {
	unsigned char pref_sync_ref;
	unsigned char wordclock_sync_check;
	unsigned char madi_sync_check;
	unsigned int system_sample_rate;
	unsigned int autosync_sample_rate;
	unsigned char system_clock_mode;
	unsigned char clock_source;
	unsigned char autosync_ref;
	unsigned char line_out;
	unsigned int passthru;
	unsigned int analog_out;
};

#define SNDRV_HDSPM_IOCTL_GET_CONFIG_INFO \
	_IOR('H', 0x41, struct hdspm_config_info)


/* get Soundcard Version */

struct hdspm_version {
	unsigned short firmware_rev;
};

#define SNDRV_HDSPM_IOCTL_GET_VERSION _IOR('H', 0x43, struct hdspm_version)


/* ------------- get Matrix Mixer IOCTL --------------- */

/* MADI mixer: 64inputs+64playback in 64outputs = 8192 => *4Byte =
 * 32768 Bytes
 */

/* organisation is 64 channelfader in a continous memory block */
/* equivalent to hardware definition, maybe for future feature of mmap of
 * them
 */
/* each of 64 outputs has 64 infader and 64 outfader: 
   Ins to Outs mixer[out].in[in], Outstreams to Outs mixer[out].pb[pb] */

#define HDSPM_MIXER_CHANNELS HDSPM_MAX_CHANNELS

struct hdspm_channelfader {
	unsigned int in[HDSPM_MIXER_CHANNELS];
	unsigned int pb[HDSPM_MIXER_CHANNELS];
};

struct hdspm_mixer {
	struct hdspm_channelfader ch[HDSPM_MIXER_CHANNELS];
};

struct hdspm_mixer_ioctl {
	struct hdspm_mixer *mixer;
};

/* use indirect access due to the limit of ioctl bit size */
#define SNDRV_HDSPM_IOCTL_GET_MIXER _IOR('H', 0x44, struct hdspm_mixer_ioctl)

/* typedefs for compatibility to user-space */
typedef struct hdspm_peak_rms hdspm_peak_rms_t;
typedef struct hdspm_config_info hdspm_config_info_t;
typedef struct hdspm_version hdspm_version_t;
typedef struct hdspm_channelfader snd_hdspm_channelfader_t;
typedef struct hdspm_mixer hdspm_mixer_t;

#endif				/* __SOUND_HDSPM_H */


enum HDSPM_LTC_FORMAT {
  format_invalid,
  fps_24,
  fps_25,
  fps_2997,
  fps_30
};

enum HDSPM_LTC_FRAME {
  frame_invalid,
  drop_frame,
  full_frame
};

enum HDSPM_LTC_INPUT_FORMAT {
  ntsc,
  pal,
  no_video
};

struct hdspm_ltc {
  unsigned int ltc;
  enum HDSPM_LTC_FORMAT format;
  enum HDSPM_LTC_FRAME frame;
  enum HDSPM_LTC_INPUT_FORMAT input_format;
};

#define SNDRV_HDSPM_IOCTL_GET_LTC _IOR('H', 0x46, struct hdspm_mixer_ioctl)


static char *texts_ports_madi[] = {
   "MADI.1",  "MADI.2",  "MADI.3",  "MADI.4",  "MADI.5",  "MADI.6",  "MADI.7",  "MADI.8",
   "MADI.9", "MADI.10", "MADI.11", "MADI.12", "MADI.13", "MADI.14", "MADI.15", "MADI.16",
  "MADI.17", "MADI.18", "MADI.19", "MADI.20", "MADI.21", "MADI.22", "MADI.23", "MADI.24",
  "MADI.25", "MADI.26", "MADI.27", "MADI.28", "MADI.29", "MADI.30", "MADI.31", "MADI.32",
  "MADI.33", "MADI.34", "MADI.35", "MADI.36", "MADI.37", "MADI.38", "MADI.39", "MADI.40",
  "MADI.41", "MADI.42", "MADI.43", "MADI.44", "MADI.45", "MADI.46", "MADI.47", "MADI.48",
  "MADI.49", "MADI.50", "MADI.51", "MADI.52", "MADI.53", "MADI.54", "MADI.55", "MADI.56",
  "MADI.57", "MADI.58", "MADI.59", "MADI.60", "MADI.61", "MADI.62", "MADI.63", "MADI.64",
};


static char *texts_ports_raydat_ss[] = { 
  "ADAT1.1", "ADAT1.2", "ADAT1.3", "ADAT1.4", "ADAT1.5", "ADAT1.6", "ADAT1.7", "ADAT1.8",
  "ADAT2.1", "ADAT2.2", "ADAT2.3", "ADAT2.4", "ADAT2.5", "ADAT2.6", "ADAT2.7", "ADAT2.8",
  "ADAT3.1", "ADAT3.2", "ADAT3.3", "ADAT3.4", "ADAT3.5", "ADAT3.6", "ADAT3.7", "ADAT3.8",
  "ADAT4.1", "ADAT4.2", "ADAT4.3", "ADAT4.4", "ADAT4.5", "ADAT4.6", "ADAT4.7", "ADAT4.8",
  "AES.L", "AES.R",
  "SPDIF.L", "SPDIF.R"
};

static char *texts_ports_raydat_ds[] = { 
  "ADAT1.1", "ADAT1.2", "ADAT1.3", "ADAT1.4", 
  "ADAT2.1", "ADAT2.2", "ADAT2.3", "ADAT2.4", 
  "ADAT3.1", "ADAT3.2", "ADAT3.3", "ADAT3.4", 
  "ADAT4.1", "ADAT4.2", "ADAT4.3", "ADAT4.4", 
  "AES.L", "AES.R",
  "SPDIF.L", "SPDIF.R"
};

static char *texts_ports_raydat_qs[] = { 
  "ADAT1.1", "ADAT1.2", 
  "ADAT2.1", "ADAT2.2", 
  "ADAT3.1", "ADAT3.2", 
  "ADAT4.1", "ADAT4.2", 
  "AES.L", "AES.R",
  "SPDIF.L", "SPDIF.R"
};


static char *texts_ports_aio_in_ss[] = { 
  "Analogue.L", "Analogue.R",
  "AES.L", "AES.R",
  "SPDIF.L", "SPDIF.R"
  "ADAT.1", "ADAT.2", "ADAT.3", "ADAT.4", "ADAT.5", "ADAT.6", "ADAT.7", "ADAT.8"
};

static char *texts_ports_aio_out_ss[] = { 
  "Analogue.L", "Analogue.R",
  "AES.L", "AES.R",
  "SPDIF.L", "SPDIF.R"
  "ADAT.1", "ADAT.2", "ADAT.3", "ADAT.4", "ADAT.5", "ADAT.6", "ADAT.7", "ADAT.8",
  "Phone.L", "Phone.R"
};

static char *texts_ports_aio_in_ds[] = { 
  "Analogue.L", "Analogue.R",
  "AES.L", "AES.R",
  "SPDIF.L", "SPDIF.R"
  "ADAT.1", "ADAT.2", "ADAT.3", "ADAT.4"
};

static char *texts_ports_aio_out_ds[] = { 
  "Analogue.L", "Analogue.R",
  "AES.L", "AES.R",
  "SPDIF.L", "SPDIF.R"
  "ADAT.1", "ADAT.2", "ADAT.3", "ADAT.4",
  "Phone.L", "Phone.R"
};

static char *texts_ports_aio_in_qs[] = { 
  "Analogue.L", "Analogue.R",
  "AES.L", "AES.R",
  "SPDIF.L", "SPDIF.R"
  "ADAT.1", "ADAT.2", "ADAT.3", "ADAT.4"
};

static char *texts_ports_aio_out_qs[] = { 
  "Analogue.L", "Analogue.R",
  "AES.L", "AES.R",
  "SPDIF.L", "SPDIF.R"
  "ADAT.1", "ADAT.2", "ADAT.3", "ADAT.4",
  "Phone.L", "Phone.R"
};

static char channel_map_unity_ss[HDSPM_MAX_CHANNELS] = {
   0,  1,  2,  3,  4,  5,  6,  7,
   8,  9, 10, 11, 12, 13, 14, 15,
  16, 17, 18, 19, 20, 21, 22, 23,
  24, 25, 26, 27, 28, 29, 30, 31,
  32, 33, 34, 35, 36, 37, 38, 39,
  40, 41, 42, 43, 44, 45, 46, 47,
  48, 49, 50, 51, 52, 53, 54, 55,
  56, 57, 58, 59, 60, 61, 62, 63
};

static char channel_map_unity_ds[HDSPM_MAX_CHANNELS] = {
   0,  2,  4,  6,  8, 10, 12, 14,
  16, 18, 20, 22, 24, 26, 28, 30,
  32, 34, 36, 38, 40, 42, 44, 46,
  48, 50, 52, 54, 56, 58, 60, 62,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,};

static char channel_map_unity_qs[HDSPM_MAX_CHANNELS] = {
   0,  4,  8, 12, 16, 20, 24, 28,
  32, 36, 40, 44, 48, 52, 56, 60
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,};


static char channel_map_raydat_ss[HDSPM_MAX_CHANNELS] = {
   4,  5,  6,  7,  8,  9, 10, 11, // ADAT 1
  12, 13, 14, 15, 16, 17, 18, 19, // ADAT 2
  20, 21, 22, 23, 24, 25, 26, 27, // ADAT 3
  28, 29, 30, 31, 32, 33, 34, 35, // ADAT 4
   0,  1,                         // AES
   2,  3,                         // SPDIF
  -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
};

static char channel_map_raydat_ds[HDSPM_MAX_CHANNELS] = {
   4, 5, 6, 7,    // ADAT 1
   8, 9, 10, 11,  // ADAT 2
  12, 13, 14, 15, // ADAT 3
  16, 17, 18, 19, // ADAT 4
   0,  1,         // AES
   2,  3,         // SPDIF
  -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
};

static char channel_map_raydat_qs[HDSPM_MAX_CHANNELS] = {
   4,  5, // ADAT 1
   6,  7, // ADAT 2
   8,  9, // ADAT 3
  10, 11, // ADAT 4
   0,  1, // AES
   2,  3, // SPDIF
  -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
};


static char channel_map_aio_in_ss[HDSPM_MAX_CHANNELS] = {
   0,  1, // line in
   8,  9, // aes in,
  10, 11, // spdif in
  12, 13, 14, 15, 16, 17, 18, 19, // ADAT in
  -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
};

static char channel_map_aio_out_ss[HDSPM_MAX_CHANNELS] = {
   0,  1, // line out
   8,  9, // aes out
  10, 11, // spdif out
  12, 13, 14, 15, 16, 17, 18, 19, // ADAT out
   6,  7, // phone out
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
};

static char channel_map_aio_in_ds[HDSPM_MAX_CHANNELS] = {
   0,  1, // line in
   8,  9, // aes in
  10, 11, // spdif in
  12, 14, 16, 18, // adat in
  -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1
};

static char channel_map_aio_out_ds[HDSPM_MAX_CHANNELS] = {
   0,  1, // line out
   8,  9, // aes out
  10, 11, // spdif out
  12, 14, 16, 18, // adat out
   6,  7, // phone out
  -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1
};


static char channel_map_aio_in_qs[HDSPM_MAX_CHANNELS] = {
   0,  1, // line in
   8,  9, // aes in
  10, 11, // spdif in
  12, 16,  // adat in
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1
};

static char channel_map_aio_out_qs[HDSPM_MAX_CHANNELS] = {
   0,  1, // line out
   8,  9, // aes out
  10, 11, // spdif out
  12, 16, // adat out
   6,  7, // phone out
  -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1
};

